<?php

require($_SERVER["DOCUMENT_ROOT"]."/bitrix/modules/main/include/prolog_before.php");

use Bitrix\Main\Config\Option;

class AirbapayAcquirinService
{
    public function __construct()
    {
        \Bitrix\Main\Loader::includeModule('airbapay.acquiring');
        \Bitrix\Main\Loader::includeModule('sale');

        $request = \Bitrix\Main\Context::getCurrent()->getRequest();

        switch ($request->getQuery('action'))
        {
            case 'update_status':
            {
                $result = $this->actionUpdateStatus();

                if (!$result->isSuccess()) {
                    \Bitrix\Sale\PaySystem\ErrorLog::add(array(
                        'ACTION' => "result status",
                        'MESSAGE' => __CLASS__ . ' : ' . json_encode($result->getErrorMessages()),
                    ));
                }

                break;
            }
        }
    }

    private function actionUpdateStatus(): \Bitrix\Main\Result
    {
        $result = new \Bitrix\Main\Result();

        $data = json_decode(file_get_contents('php://input'), true);

        if ($data['type'] === 'PAYMENT') {
            $publickey = 'MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAv0pBoPtX5i+FGS16Bqfy'.
            'WI0xNzq/C3t4GDrySZQYOaY7lpaWaZM9sy8mB20PCMKoSkHhYK9Gp3wtmheFJIG7'.
            'd2Shxcj0+VhyvgzhBqdBfx3/CjTEj5OzVtuVhlkVZacrpArrj4GIAk3u4c6Ut7O/'.
            'Hzop/cGzNxKgUXgq31VWP9CTYVFiUe8ArGF7dTuIhN6eVaLBhaViYt32HSKctIt4'.
            'UQCn1azU+8aEkz0qbFXvToXalaIQu8Pzir6epYOlzxWdsmKgtkRH1eGQJRbITJRX'.
            'xK7E+ePi8R9lAYGTlnBJhJNCEp3g7+GgFy91pWqp1tmE5L2VAaHzVXO06mh0Y6J5'.
            'ZwIDAQAB';
            
            preg_replace("(.{64})", "\1\r\n", $publickey);
            $publickey = "-----BEGIN PUBLIC KEY-----\r\n".$publickey."\r\n-----END PUBLIC KEY-----";

            $pkey = openssl_pkey_get_public($publickey);

            $str = $data['id'] . $data['invoice_id'] . $data['amount'] . $data['currency'] . $data['status'] . $data['description'];

            $verify = openssl_verify($str, base64_decode($data['sign']), $pkey, OPENSSL_ALGO_SHA256);

            if ($verify !== 1) {
                $result->addError(new \Bitrix\Main\Error('verify'));
                return $result;
            }

            $arOrder = \Airbapay\Acquiring\OrderTable::getRow([
                'filter' => [
                    '=AIRBAPAY_ID' => $data['id'],
                    '=AIRBAPAY_INVOICE_ID' => $data['invoice_id'],
                ]
            ]);

            if (!$arOrder) {
                $result->addError(new \Bitrix\Main\Error('arOrder'));
                return $result;
            }

            $order = \Bitrix\Sale\Order::load($arOrder['ORDER_ID']);
            $paymentCollection = $order->getPaymentCollection();
            $payment = $paymentCollection->getItemById($arOrder['PAYMENT_ID']);

            if ($payment->getSum() !== floatval($data['amount'])) {
                $result->addError(new \Bitrix\Main\Error('amount'));
                return $result;
            }

            if ($order->isCanceled()) {
                $result->addError(new \Bitrix\Main\Error('order canceled'));
                return $result;
            }

            if (in_array($data['status'], [
                \Airbapay\Acquiring\HelperBitrix::STATUS_SUCCESS,
                \Airbapay\Acquiring\HelperBitrix::STATUS_AUTH,
            ])) {
                if (Bitrix\Sale\OrderStatus::isAllowPay($order->getField('STATUS_ID'))) {
                    if (!$payment->isPaid()) {
                        $paidResult = $payment->setPaid('Y');
                        if (!$paidResult->isSuccess()) {
                            $result->addError(new \Bitrix\Main\Error('set paid'));
                            return $result;
                        }

                        $saveResult = $order->save();

                        if (!$saveResult->isSuccess()) {
                            $result->addError(new \Bitrix\Main\Error('order save'));
                            return $result;
                        }
                    }
                }
            }

            \Airbapay\Acquiring\OrderTable::update($arOrder['ID'], [
                'STATUS' => $data['status'],
            ]);
        }

        return $result;
    }
}

new AirbapayAcquirinService();