<?php

use Bitrix\Main\Application;
use Bitrix\Main\Loader;
use Bitrix\Main\Localization\Loc;
use Bitrix\Main\ModuleManager;
use Bitrix\Main\EventManager;
use Bitrix\Main\Config\Option;

Loc::loadMessages(__FILE__);

class airbapay_acquiring extends CModule
{
    public function __construct()
    {
        $arModuleVersion = array();
        
        include __DIR__ . '/version.php';

        if (is_array($arModuleVersion) && array_key_exists('VERSION', $arModuleVersion))
        {
            $this->MODULE_VERSION = $arModuleVersion['VERSION'];
            $this->MODULE_VERSION_DATE = $arModuleVersion['VERSION_DATE'];
        }
        
        $this->MODULE_ID = 'airbapay.acquiring';
        $this->MODULE_NAME = 'Airbapay Acquiring';
        $this->MODULE_DESCRIPTION = 'Airbapay Acquiring';
        $this->MODULE_GROUP_RIGHTS = 'N';
        $this->PARTNER_NAME = 'Airbapay';
        $this->PARTNER_URI = 'https://airbapay.kz';
    }

    public function doInstall()
    {
        ModuleManager::registerModule($this->MODULE_ID);

        $this->installFiles();
        
        $this->registerEvents();

        $this->installDB();
    }

    public function registerEvents()
    {
        $eventManager = \Bitrix\Main\EventManager::getInstance();

        $eventManager->registerEventHandler("sale", "OnSaleOrderSaved", $this->MODULE_ID, "\\Airbapay\\Acquiring\\Main", "onSaleOrderSaved");
    }

    public function doUninstall()
    {
        global $DOCUMENT_ROOT, $APPLICATION, $step;
        $step = intval($step);

        \CAgent::RemoveModuleAgents($this->MODULE_ID);

        if ($step === 2) {
            $this->uninstallFiles();
            // $this->uninstallDB();
        }

        $this->unRegisterEvents();

        ModuleManager::unRegisterModule($this->MODULE_ID);
    }

    public function unRegisterEvents()
    {
        $eventManager = \Bitrix\Main\EventManager::getInstance();

        $eventManager->unRegisterEventHandler("sale", "OnSaleOrderSaved", $this->MODULE_ID, "\\Airbapay\\Acquiring\\Main", "onSaleOrderSaved");
    }

    public function installDB()
    {
        if (Loader::includeModule($this->MODULE_ID))
        {
            global $APPLICATION, $DB, $errors;

            if (!$DB->Query("SELECT 'x' FROM " . \Airbapay\Acquiring\OrderTable::getTableName(), true)) {
                \Airbapay\Acquiring\OrderTable::getEntity()->createDbTable();
            }
        }
    }

    public function uninstallDB()
    {
        if (Loader::includeModule($this->MODULE_ID))
        {
            $connection = Application::getInstance()->getConnection();
            $connection->dropTable(\Airbapay\Acquiring\OrderTable::getTableName());
        }
    }

    function installFiles()
	{
        $path2user_ps_files = \Bitrix\Main\Config\Option::get('sale', 'path2user_ps_files', BX_ROOT . '/php_interface/include/sale_payment/');

        $salePaymentPath = $_SERVER["DOCUMENT_ROOT"] . $path2user_ps_files;
        if (!file_exists($salePaymentPath)) {
            mkdir($salePaymentPath, 0775);
        }

        copyDirFiles(
			__DIR__ . '/sale_payment_path/',
			$salePaymentPath,
			true, true
		);

        copyDirFiles(
			__DIR__ . '/bitrix/',
			$_SERVER["DOCUMENT_ROOT"] . '/bitrix/',
			true, true
		);

		return true;
    }
    
    function uninstallFiles()
	{
        $path2user_ps_files = \Bitrix\Main\Config\Option::get('sale', 'path2user_ps_files', BX_ROOT . '/php_interface/include/sale_payment/');

        $salePaymentPath = $_SERVER["DOCUMENT_ROOT"] . $path2user_ps_files;

        if (file_exists($salePaymentPath . 'airbapayacquiring/')) {
            \Bitrix\Main\IO\Directory::deleteDirectory($path2user_ps_files . 'airbapayacquiring/');
        }

        \Bitrix\Main\IO\File::deleteFile($_SERVER["DOCUMENT_ROOT"] . '/bitrix/images/sale/sale_payments/airbapayacquiring.png');

        \Bitrix\Main\IO\File::deleteFile($_SERVER["DOCUMENT_ROOT"] . '/bitrix/tools/airbapay_acquiring.php');
        
		return true;
	}
}
