<?php

namespace Airbapay\Acquiring;

use Bitrix\Main\Web\HttpClient;

class HelperBitrix extends Helper
{
    public function getAccessToken($paymentId): array
    {
        if ($paymentId) {
            $tokenKey = $paymentId;
        } else {
            $tokenKey = '-';
        }

        if (!isset($this->accessToken[$tokenKey])) {
            $url = $this->getBaseUrl();

            $data = [
                "user" => $this->user,
                "password" => $this->password,
                "terminal_id" => $this->terminalId,
            ];

            if ($paymentId) {
                $data['payment_id'] = $paymentId;
            }
    
            $httpClient = new HttpClient();
    
            $httpClient->setHeader('Content-Type', 'application/json');
    
            $response = $httpClient->post($url . 'auth/sign-in', json_encode($data));
            if ($response === false) {
                $errors = $httpClient->getError();
                // foreach ($errors as $code => $message)
                // {
                // 	$result->addError(new Main\Error($message, $code));
                // }
    
                // return $result;
            }
    
            $response = json_decode($response, 1);
    
            if (isset($response['error'])) {
                return [
                    'status' => 'error',
                    'errors' => [
                        [
                            'message' => $response['error']['message'],
                            'code' => $response['error']['code'],
                        ],
                    ],
                ];
            }
    
            if (!isset($response['access_token'])) {
                return [
                    'status' => 'error',
                    'errors' => [
                        [
                            'message' => 'Not isset access token',
                            'code' => null,
                        ],
                    ],
                ];
            }

            $this->accessToken[$tokenKey] = $response['access_token'];
        }

        return [
            'status' => 'success',
            'data' => [
                'accessToken' => $this->accessToken[$tokenKey],
            ],
        ];
    }

    public function apiQuery(string $action, array $data = null, array $opt = []): array
    {
        $rsToken = $this->getAccessToken($opt['payment_id'] ?? false);

        if ($rsToken['status'] !== 'success') {
            return $rsToken;
        }

        $accessToken = $rsToken['data']['accessToken'];

        $baseUrl = $this->getBaseUrl();
        
        $url = $baseUrl . $action;

        $httpClient = new HttpClient();

        $httpClient->setHeader('Content-Type', 'application/json');
        $httpClient->setHeader('Authorization', 'Bearer ' . $accessToken);

        if (($opt['method'] ?? false)) {
            if ($data !== null) {
                if (!$data) {
                    $body = '{}';
                } else {
                    $body = json_encode($data);
                }

                $response = $httpClient->query($opt['method'], $url, $body);
            } else {
                $response = $httpClient->query($opt['method'], $url);
            }

            if ($response) {
                $response = $httpClient->getResult();
            }
        } else {
            if ($data !== null) {
                $response = $httpClient->post($url, json_encode($data));
            } else {
                $response = $httpClient->get($url);
            }
        }
        
		if ($response === false) {
			$errors = $httpClient->getError();
			// foreach ($errors as $code => $message)
			// {
			// 	$result->addError(new Main\Error($message, $code));
			// }

			// return $result;
		}

        $response = json_decode($response, 1);

        if (isset($response['error'])) {
			return [
                'status' => 'error',
                'errors' => [
                    [
                        'message' => $response['error']['message'],
                        'code' => $response['error']['code'],
                    ],
                ],
            ];
		}

        return [
            'status' => 'success',
            'data' => [
                'response' => $response,
            ],
        ];
    }
}