<?php

namespace Airbapay\Acquiring;

use Bitrix\Sale\PaySystem;

class Main
{
    private static $disallowOnSaleOrderSaved = false;

    public static function onSaleOrderSaved(\Bitrix\Main\Event $event)
    {
        if (self::$disallowOnSaleOrderSaved !== false) {
            return;
        }

        self::$disallowOnSaleOrderSaved = true;

        $order = $event->getParameter("ENTITY");
        $oldValues = $event->getParameter("VALUES");
        $isNew = $event->getParameter("IS_NEW");

        $paymentCollection = $order->getPaymentCollection();

        foreach ($paymentCollection as $payment) {
            $paysystem = $payment->getPaySystem();

            if ($paysystem->getField('ACTION_FILE') === 'airbapayacquiring') {
                $statusDebitingFunds = \Bitrix\Sale\BusinessValue::get(
                    "AIRBAPAY_ACQUIRING_ORDER_STATUS_DEBITING_FUNDS",
                    "PAYSYSTEM_" . $paysystem->getField('ID'),
                    $order->getPersonTypeId(),
                );

                if ($statusDebitingFunds && isset($oldValues['STATUS_ID'])) {
                    if ($order->getField('STATUS_ID') !== $oldValues['STATUS_ID'] && $order->getField('STATUS_ID') === $statusDebitingFunds) {
                        $arOrder = \Airbapay\Acquiring\OrderTable::getRow([
                            'filter' => [
                                '=PAYMENT_ID' => $payment->getId(),
                                '=STATUS' => \Airbapay\Acquiring\HelperBitrix::STATUS_AUTH,
                            ]
                        ]);

                        if ($arOrder) {
                            $helper = self::getHelper($paysystem->getField('ID'), $order->getPersonTypeId());

                            $data = [
                                'amount' => $payment->getSum(),
                            ];
    
                            $rsQuery = $helper->apiQuery('payments/charge', $data, [
                                'method' => 'PUT',
                                'payment_id' => $arOrder['AIRBAPAY_ID'],
                            ]);
    
                            if ($rsQuery['status'] !== 'success') {
                                foreach ($rsQuery['errors'] as $error) {
                                    PaySystem\ErrorLog::add(array(
                                        'ACTION' => "payments/charge",
                                        'MESSAGE' => __CLASS__ . ' : payments/charge : ' . $error['message'] . '[' . $error['code'] . ']',
                                    ));
                                }

                                return;
                            }

                            $status = ($rsQuery['data']['response']['status'] ?? false);
    
                            if ($status !== \Airbapay\Acquiring\HelperBitrix::STATUS_SUCCESS) {
                                PaySystem\ErrorLog::add(array(
                                    'ACTION' => "get success status",
                                    'MESSAGE' => __CLASS__ . ' : get success status',
                                ));

                                return;
                            }

                            
                            if ($status === \Airbapay\Acquiring\HelperBitrix::STATUS_SUCCESS) {
                                if (!\Bitrix\Sale\OrderStatus::isAllowPay($order->getField('STATUS_ID'))) {
                                    PaySystem\ErrorLog::add(array(
                                        'ACTION' => "order not allow pay",
                                        'MESSAGE' => __CLASS__ . ' : order not allow pay',
                                    ));

                                    return;
                                }

                                if ($payment->isPaid()) {
                                    PaySystem\ErrorLog::add(array(
                                        'ACTION' => "payment is paid",
                                        'MESSAGE' => __CLASS__ . ' : payment is paid',
                                    ));

                                    return;
                                }

                                if ($payment->isReturn()) {
                                    PaySystem\ErrorLog::add(array(
                                        'ACTION' => "payment is return",
                                        'MESSAGE' => __CLASS__ . ' : payment is return',
                                    ));

                                    return;
                                }

                                $paidResult = $payment->setPaid('Y');
                                if (!$paidResult->isSuccess()) {
                                    PaySystem\ErrorLog::add(array(
                                        'ACTION' => "setPaid error",
                                        'MESSAGE' => __CLASS__ . ' : setPaid error',
                                    ));

                                    return;
                                }

                                $saveResult = $order->save();

                                if (!$saveResult->isSuccess()) {
                                    PaySystem\ErrorLog::add(array(
                                        'ACTION' => "save order error",
                                        'MESSAGE' => __CLASS__ . ' : save order error',
                                    ));

                                    return;
                                }
                            }

                            \Airbapay\Acquiring\OrderTable::update($arOrder['ID'], [
                                'STATUS' => $status,
                            ]);
                        }
                    }
                }
            }
        }

        self::$disallowOnSaleOrderSaved = false;
    }

    public static function getHelper($paysystemId, $personTypeId)
    {
        $isTestModeParam = \Bitrix\Sale\BusinessValue::get(
            "AIRBAPAY_ACQUIRING_TEST_MODE",
            "PAYSYSTEM_" . $paysystemId,
            $personTypeId,
        );

        $user = \Bitrix\Sale\BusinessValue::get(
            "AIRBAPAY_ACQUIRING_USER",
            "PAYSYSTEM_" . $paysystemId,
            $personTypeId,
        );

        $password = \Bitrix\Sale\BusinessValue::get(
            "AIRBAPAY_ACQUIRING_PASSWORD",
            "PAYSYSTEM_" . $paysystemId,
            $personTypeId,
        );

        $terminalId = \Bitrix\Sale\BusinessValue::get(
            "AIRBAPAY_ACQUIRING_TERMINAL_ID",
            "PAYSYSTEM_" . $paysystemId,
            $personTypeId,
        );

        return new \Airbapay\Acquiring\HelperBitrix([
            'isTestMode' => $isTestModeParam === 'Y',
            'user' => $user,
            'password' => $password,
            'terminalId' => $terminalId,
        ]);
    }
}